% This function is used to extract the value from flattened map and transform back
% to true bold activated value.
% By Yuxiang, August 23, 2017

% Test version
% Version for Train and Test dataset

Path = '/autofs/space/bidlin7_001/users/GAN/GSP';
sublist = textread([Path '/Lists/list_train_and_test.txt'], '%s');

% ---- Change Start ----
Subject_no = 81:100;
Dataset = 'GSP_lh_Test20';
ImageSet = ['/autofs/space/bidlin7_001/users/GAN/GSP/Images/' Dataset];
Hemispheres = {'lh'};
Bolds = {'013','014'};
Frames = 1:120;
% ---- Change Stop ----

vertex_location = load([Path '/Scripts/VertexLocation/vertex_location.mat']);
OriginalBoldSet = ['/autofs/space/bidlin7_001/users/GAN/GSP/Images/mgh_files/' Dataset];
mkdir([Path '/ReconstructedBOLD/' Dataset]);
mkdir([Path '/ReconstructedBOLD/' Dataset '/SingleFrames']);

for s = Subject_no
    if s<=80
        set_name = 'Train80';
    else
        set_name = 'Test20';
    end
    
    subject = sublist{s*2-1}
    
    for h = 1:length(Hemispheres)
        hemishpere = Hemispheres{h};
        
        Reconstructed_BOLD = zeros(2562,length(Bolds).*length(Frames));
        Real_BOLD = Reconstructed_BOLD;
        
        for b = 1:length(Bolds)
            bold = Bolds{b};
            for f = Frames
                frame = Frames(f);
                
                % Load flattened image
                flattened_image = double(imread([ImageSet '/' set_name '_' num2str(s) '_' subject '_' hemishpere '_' bold '_' num2str(frame) '.tiff']));
                if size(flattened_image,1) == 600
                    flattened_image = flattened_image(51:550,51:550,:);
                end
                % Load orginal BOLD activation
                bold_activation = load_mgh([OriginalBoldSet '/' set_name '_' num2str(s) '_' subject '_' hemishpere '_' bold '_' num2str(frame) '.mgh']);
                
                transformed_bold = zeros(size(bold_activation,1),4); % store rgb and transformed value
                for nn = 1:size(bold_activation,1)
                    the_location = vertex_location.vertex_location(nn,1:2);
                    if the_location(1) == -50
                        transformed_bold(nn,1:3) = [-2 -2 -2]; % [0 0 0]
                    else
                        transformed_bold(nn,1:3) = flattened_image(the_location(1),the_location(2),:);
                    end
                end
                
%                 % Compare the value of original BOLD and flattened map, for test
%                 T = [bold_activation transformed_bold];
%                 [Y,I] = sort(T(:,1));
%                 T2 = T(I,:);
%                 T3 = T2;
%                 T3(T3(:,2)==-2,:) = [];
%                 figure;hold on; plot(T3(:,1).*140+140,'k'); plot(T3(:,2),'r'); plot(T3(:,3),'g'); plot(T3(:,4),'b'); hold off;

                
                % Change the green value to negative if r < b
                temp_g = transformed_bold(transformed_bold(:,1)>0,2);
                min_g = min(temp_g);
                transformed_bold(:,2) = transformed_bold(:,2) - min_g;
                transformed_bold((transformed_bold(:,1)<transformed_bold(:,3)),2) = - transformed_bold((transformed_bold(:,1)<transformed_bold(:,3)),2);
                
                transformed_bold(transformed_bold(:,2)>=0,4) = transformed_bold(transformed_bold(:,2)>=0,2).*min(0.5,max(bold_activation))./(max(transformed_bold(:,2)));
                transformed_bold(transformed_bold(:,2)<0,4) = transformed_bold(transformed_bold(:,2)<0,2).*min(0.5,(0 - min(bold_activation)))./(0 - min(transformed_bold(:,2)));
                transformed_bold(transformed_bold(:,1)==-2,4) = -2;
                
%                 % For test
%                 transformed_bold2 = transformed_bold;
%                 transformed_bold2(transformed_bold2(:,1)==-2,:)=[];
%                 bold_activation2 = bold_activation;
%                 bold_activation2(transformed_bold(:,1)==-2,:)=[];
%                 corr(transformed_bold2(:,4),bold_activation2)
%                 
%                 T = [bold_activation2 transformed_bold2];
%                 [Y,I] = sort(T(:,1));
%                 T = T(I,:);
%                 figure;hold on; plot(T(:,1),'k'); plot(T(:,5),'g');
                Reconstructed_BOLD(:,(b-1)*120+f) = transformed_bold(:,4);
                Real_BOLD(:,(b-1)*120+f) = bold_activation;
                
                save_mgh(Reconstructed_BOLD, [Path '/ReconstructedBOLD/' Dataset '/SingleFrames/' set_name '_' num2str(s) '_' subject '_' hemishpere '_' bold '_' num2str(frame) '.mgh'], eye(4));
            end
        end
        save_mgh(Reconstructed_BOLD, [Path '/ReconstructedBOLD/' Dataset '/' Dataset '_' num2str(s) '_' subject '_ReconstructedBOLD.mgh'], eye(4));
        save_mgh(Real_BOLD, [Path '/ReconstructedBOLD/' Dataset '/' Dataset '_' num2str(s) '_' subject '_RealBOLD.mgh'], eye(4));
    end
end
